//
//  AcademicCalendarView.swift
//  TeachMate
//
//  Created by Hongli on 2025/3/12.
//

import SwiftUI

struct AcademicCalendarView: View {
    let semester: Semester
    
    // 布局常量
    private let weekdays = ["一", "二", "三", "四", "五", "六", "日"]
    private let cellHeight: CGFloat = 35
    private let weekColumnWidth: CGFloat = 50
    private let dayCellWidth: CGFloat = 60
    private let padding: CGFloat = 10
    
    var body: some View {
        ScrollView {
            calendarGrid
                .padding(padding)
        }
        .frame(maxWidth: .infinity, maxHeight: .infinity)
    }
    
    // 日历网格
    private var calendarGrid: some View {
        VStack(spacing: 0) {
            // 表头行
            headerRow
            
            // 日期行
            ForEach(0..<semester.weeksCount, id: \.self) { weekIndex in
                weekRow(weekIndex: weekIndex)
            }
        }
    }
    
    // 表头行
    private var headerRow: some View {
        HStack(spacing: 0) {
            // 周次表头
            Text("周次")
                .font(.headline)
                .frame(width: weekColumnWidth, height: cellHeight)
                .background(Color.gray.opacity(0.3))
                .border(Color.gray.opacity(0.5), width: 0.5)
            
            // 星期表头
            ForEach(weekdays, id: \.self) { weekday in
                Text(weekday)
                    .font(.headline)
                    .frame(width: dayCellWidth, height: cellHeight)
                    .background(Color.gray.opacity(0.3))
                    .border(Color.gray.opacity(0.5), width: 0.5)
            }
        }
    }
    
    // 周行
    private func weekRow(weekIndex: Int) -> some View {
        let dates = weekDates(for: weekIndex)
        
        return HStack(spacing: 0) {
            // 周数
            Text("\(weekIndex + 1)")
                .font(.headline)
                .frame(width: weekColumnWidth, height: cellHeight)
                .background(Color.gray.opacity(0.3))
                .border(Color.gray.opacity(0.5), width: 0.5)
            
            // 日期单元格
            ForEach(0..<7, id: \.self) { dayIndex in
                dateCell(date: dates[dayIndex], isWeekend: dayIndex >= 5)
            }
        }
    }
    
    // 日期单元格
    private func dateCell(date: Date?, isWeekend: Bool) -> some View {
        Group {
            if let date = date {
                Text(formatDate(date))
                    .frame(width: dayCellWidth, height: cellHeight)
                    .background(isWeekend ? Color.gray.opacity(0.2) : Color.white)
                    .border(Color.gray.opacity(0.5), width: 0.5)
            } else {
                Text("")
                    .frame(width: dayCellWidth, height: cellHeight)
                    .background(Color.gray.opacity(0.1))
                    .border(Color.gray.opacity(0.5), width: 0.5)
            }
        }
    }
    
    // 格式化日期显示
    private func formatDate(_ date: Date) -> String {
        let formatter = DateFormatter()
        let calendar = Calendar.current
        
        // 如果是月份的第一天，显示"月日"格式
        if calendar.component(.day, from: date) == 1 {
            formatter.dateFormat = "M月d日"
            return formatter.string(from: date)
        }
        
        // 否则只显示日期
        formatter.dateFormat = "d"
        return formatter.string(from: date)
    }
    
    // 获取指定周的日期数组
    private func weekDates(for weekIndex: Int) -> [Date?] {
        let calendar = Calendar.current
        
        // 计算该周的第一天
        let weekStartDate = calendar.date(byAdding: .day, value: weekIndex * 7, to: firstMondayDate)!
        
        // 创建该周的日期数组
        var dates: [Date?] = []
        for dayIndex in 0..<7 {
            let currentDate = calendar.date(byAdding: .day, value: dayIndex, to: weekStartDate)!
            
            // 检查日期是否在学期范围内（包含结束日期）
            // 使用 Calendar 的 compare 方法进行更准确的日期比较
            let startComparison = calendar.compare(currentDate, to: semester.startDate, toGranularity: .day)
            let endComparison = calendar.compare(currentDate, to: semester.endDate, toGranularity: .day)
            
            if (startComparison == .orderedSame || startComparison == .orderedDescending) &&
               (endComparison == .orderedSame || endComparison == .orderedAscending) {
                dates.append(currentDate)
            } else {
                dates.append(nil)
            }
        }
        
        return dates
    }
    
    // 计算学期第一周的周一日期
    private var firstMondayDate: Date {
        let calendar = Calendar.current
        let startDate = semester.startDate
        
        // 获取开始日期是周几（1=周一，2=周二，...，7=周日）
        let weekday = calendar.component(.weekday, from: startDate)
        // 转换为中国周历（周一是1，周日是7）
        let chineseWeekday = weekday == 1 ? 7 : weekday - 1
        
        // 如果不是周一，则回退到该周的周一
        if chineseWeekday > 1 {
            return calendar.date(byAdding: .day, value: -(chineseWeekday - 1), to: startDate)!
        }
        
        return startDate
    }
}

#Preview {
    AcademicCalendarView(semester: PreviewData.createSampleSemester())
}
