//
//  CourseScheduleView.swift
//  TeachMate
//
//  Created by Hongli on 2025/3/13.
//

import SwiftUI
import SwiftData

struct CourseScheduleView: View {
    let semester: Semester
    @State private var currentWeek: Int = 1
    @State private var showingAddCourse = false
    @State private var selectedTimeSlot: (weekday: Int, timeSlot: Int)? = nil
    
    @Environment(\.modelContext) private var modelContext
    @Query private var courses: [CourseModel]
    
    // 星期标题
    private let weekdays = ["一", "二", "三", "四", "五", "六", "日"]
    
    // 布局常量
    private let timeColumnWidth: CGFloat = 100
    private let dayCellWidth: CGFloat = 120
    private let cellHeight: CGFloat = 100
    private let headerHeight: CGFloat = 40
    
    // 初始化查询
    init(semester: Semester) {
        self.semester = semester
        
        // 不使用谓词，而是在视图中过滤课程
        self._courses = Query()
    }
    
    // 过滤后的课程列表
    private var filteredCourses: [CourseModel] {
        courses.filter { $0.semester?.id == semester.id }
    }
    
    // 判断是否有周六日的课程
    private var hasWeekendCourses: Bool {
        filteredCourses.contains { course in
            course.sessions.contains { session in
                session.weekday == 6 || session.weekday == 7
            }
        }
    }
    
    // 判断是否有9/10节的课程
    private var hasLateTimeslotCourses: Bool {
        filteredCourses.contains { course in
            course.sessions.contains { session in
                session.timeSlot == 5 // 5 对应 9/10 节
            }
        }
    }
    
    // 获取应该显示的星期
    private var displayWeekdays: [Int] {
        hasWeekendCourses ? Array(1...7) : Array(1...5)
    }
    
    // 获取应该显示的时间段
    private var displayTimeSlots: [TimeSlot] {
        hasLateTimeslotCourses ? TimeSlot.defaultSlots : TimeSlot.defaultSlots.filter { $0.id != 5 }
    }
    
    // 获取特定单元格的课程
    private func getCoursesForCell(weekday: Int, timeSlot: Int) -> [CourseModel] {
        return filteredCourses.filter { course in
            course.sessions.contains { session in
                session.weekday == weekday && session.timeSlot == timeSlot
            }
        }
    }
    
    var body: some View {
        VStack(spacing: 0) {
            // 周次选择器
            weekSelector
                .padding(.horizontal)
                .padding(.vertical, 8)
            
            // 课程表
            ScrollView([.horizontal, .vertical]) {
                VStack(spacing: 0) {
                    // 表头行
                    headerRow
                    
                    // 课程行
                    ForEach(displayTimeSlots) { timeSlot in
                        courseRow(timeSlot: timeSlot)
                    }
                }
                .padding()
            }
        }
        .frame(maxWidth: .infinity, maxHeight: .infinity)
        .onAppear {
            // 设置当前周
            if let weekNumber = semester.currentWeekNumber() {
                currentWeek = weekNumber
            }
            
            // 如果没有课程数据，添加示例数据
            if filteredCourses.isEmpty {
                addSampleCourses()
            }
        }
        .sheet(isPresented: $showingAddCourse) {
            if let selected = selectedTimeSlot {
                AddCourseView(semester: semester, weekday: selected.weekday, timeSlot: selected.timeSlot)
            }
        }
    }
    
    // 周次选择器
    private var weekSelector: some View {
        HStack {
            Button(action: {
                if currentWeek > 1 {
                    currentWeek -= 1
                }
            }) {
                Image(systemName: "chevron.left")
            }
            .disabled(currentWeek <= 1)
            
            Spacer()
            
            Picker("周次", selection: $currentWeek) {
                ForEach(1...semester.weeksCount, id: \.self) { week in
                    Text("第\(week)周").tag(week)
                }
            }
            .pickerStyle(.menu)
            .frame(width: 120)
            
            Spacer()
            
            Button(action: {
                if currentWeek < semester.weeksCount {
                    currentWeek += 1
                }
            }) {
                Image(systemName: "chevron.right")
            }
            .disabled(currentWeek >= semester.weeksCount)
        }
    }
    
    // 表头行
    private var headerRow: some View {
        HStack(spacing: 0) {
            // 左上角空白单元格
            Text("第\(currentWeek)周")
                .font(.headline)
                .frame(width: timeColumnWidth, height: headerHeight)
                .background(Color.gray.opacity(0.3))
                .border(Color.gray.opacity(0.5), width: 0.5)
            
            // 星期表头
            ForEach(displayWeekdays, id: \.self) { weekday in
                Text(weekdays[weekday - 1])
                    .font(.headline)
                    .frame(width: dayCellWidth, height: headerHeight)
                    .background(Color.gray.opacity(0.3))
                    .border(Color.gray.opacity(0.5), width: 0.5)
            }
        }
    }
    
    // 课程行
    private func courseRow(timeSlot: TimeSlot) -> some View {
        HStack(spacing: 0) {
            // 时间段信息
            VStack(alignment: .center, spacing: 2) {
                Text(timeSlot.name)
                    .font(.system(size: 13, weight: .medium))
                Text(timeSlot.timeRange)
                    .font(.system(size: 12))
                    .foregroundColor(.secondary)
            }
            .frame(width: timeColumnWidth, height: cellHeight)
            .background(Color.gray.opacity(0.1))
            .border(Color.gray.opacity(0.5), width: 0.5)
            
            // 每天的课程单元格
            ForEach(displayWeekdays, id: \.self) { weekday in
                courseCell(weekday: weekday, timeSlot: timeSlot.id)
            }
        }
    }
    
    // 课程单元格
    private func courseCell(weekday: Int, timeSlot: Int) -> some View {
        let coursesForCell = getCoursesForCell(weekday: weekday, timeSlot: timeSlot)
        
        return Group {
            if let course = coursesForCell.first {
                courseDisplayView(course: course, weekday: weekday, timeSlot: timeSlot)
                    .padding(5)
                    .frame(width: dayCellWidth, height: cellHeight)
                    .background(course.backgroundColor)
                    .border(Color.gray.opacity(0.5), width: 0.5)
                    .contextMenu {
                        Button(role: .destructive) {
                            deleteCourse(course)
                        } label: {
                            Label("删除课程", systemImage: "trash")
                        }
                    }
            } else {
                Text("")
                    .frame(width: dayCellWidth, height: cellHeight)
                    .background(Color.white)
                    .border(Color.gray.opacity(0.5), width: 0.5)
                    .onTapGesture {
                        selectedTimeSlot = (weekday, timeSlot)
                        showingAddCourse = true
                    }
            }
        }
    }
    
    // 课程显示视图
    private func courseDisplayView(course: CourseModel, weekday: Int, timeSlot: Int) -> some View {
        // 获取当前单元格对应的课程会话
        let session = course.sessions.first { $0.weekday == weekday && $0.timeSlot == timeSlot }
        
        return VStack(spacing: 2) {
            Text(course.name)
                .font(.system(size: 11))
                .foregroundColor(.black)
                .lineLimit(1)
            Text(session?.schoolCalss ?? "未知班级")
                .font(.system(size: 10))
                .foregroundColor(.black)
            
            if let location = session?.location {
                Text(location)
                    .font(.system(size: 10))
                    .foregroundColor(.black)
                    .lineLimit(1)
            }
        }
    }
    
    // 添加示例课程数据
    private func addSampleCourses() {
        // 使用 PreviewData 中的示例数据
        let container = PreviewData.createContainer()
        let fetchDescriptor = FetchDescriptor<CourseModel>()
        let sampleCourses = try? container.mainContext.fetch(fetchDescriptor)
            .filter { $0.semester?.id != semester.id } // 过滤掉已经属于当前学期的课程
        
        
        for course in sampleCourses ?? [] {
            // 创建新课程，复制原课程的属性
            let newCourse = CourseModel(name: course.name, colorHex: course.colorHex)

            
            // 复制课时信息
            for session in course.sessions {
                newCourse.addSession(
                    weekday: session.weekday,
                    timeSlot: session.timeSlot,
                    location: session.location,
                    schoolClass: session.schoolCalss
                )
            }
            
            // 关联到当前学期并插入
            newCourse.semester = semester
            modelContext.insert(newCourse)
        }
    }
    
    // 删除课程
    private func deleteCourse(_ course: CourseModel) {
        modelContext.delete(course)
    }
}

// 添加课程视图
struct AddCourseView: View {
    @Environment(\.dismiss) private var dismiss
    @Environment(\.modelContext) private var modelContext
    
    let semester: Semester
    let weekday: Int
    let timeSlot: Int
    
    @State private var name: String = ""
    @State private var location: String = ""
    @State private var schoolClass: String = ""
    @State private var isNew: Bool = false
    @State private var selectedColor: String = "#FCF3CF" // 默认黄色
    
    // 可选颜色
    private let colorOptions = [
        "#FCF3CF", // 黄色
        "#D6EAF8", // 蓝色
        "#FADBD8", // 红色
        "#D5F5E3", // 绿色
        "#E8DAEF"  // 紫色
    ]
    
    var body: some View {
        NavigationView {
            Form {
                Section(header: Text("课程信息")) {
                    TextField("课程名称", text: $name)
                    TextField("班级", text: $schoolClass)
                    TextField("上课地点", text: $location)
                    Toggle("标记为新课程", isOn: $isNew)
                }
                
                Section(header: Text("颜色")) {
                    HStack {
                        ForEach(colorOptions, id: \.self) { colorHex in
                            Circle()
                                .fill(Color(hex: colorHex) ?? .gray)
                                .frame(width: 30, height: 30)
                                .overlay(
                                    Circle()
                                        .stroke(selectedColor == colorHex ? Color.black : Color.clear, lineWidth: 2)
                                )
                                .onTapGesture {
                                    selectedColor = colorHex
                                }
                                .padding(5)
                        }
                    }
                }
                
                Section {
                    Button("添加课程") {
                        addCourse()
                    }
                    .frame(maxWidth: .infinity, alignment: .center)
                    .disabled(name.isEmpty || location.isEmpty || location.isEmpty)
                }
            }
            .navigationTitle("添加课程")
            .toolbar {
                ToolbarItem(placement: .cancellationAction) {
                    Button("取消") {
                        dismiss()
                    }
                }
            }
        }
    }
    
    private func addCourse() {
        let course = CourseModel(
            name: name,
            location: location,
            weekday: weekday,
            timeSlot: timeSlot,
            colorHex: selectedColor,
            isNew: isNew,
            schoolClass: schoolClass
        )
        
        course.semester = semester
        modelContext.insert(course)
        dismiss()
    }
}

#Preview {
    CourseScheduleView(semester: PreviewData.createSampleSemester())
        .modelContainer(PreviewData.createContainer())
}
