//
//  SessionFormView.swift
//  TeachMate
//
//  Created by Hongli on 2025/3/18.
//

import SwiftUI
import SwiftData

// 课时弹窗
struct SessionFormView: View {
    @Environment(\.dismiss) private var dismiss
    @Environment(\.modelContext) private var modelContext
    @Environment(\.colorScheme) private var colorScheme
    
    enum FormMode {
        case add
        case edit(ClassSession)
    }
    
    let mode: FormMode
    let course: CourseModel
    
    @State private var weekday: Int = 1
    @State private var timeSlot: Int = 1
    @State private var location: String = ""
    @State private var schoolClass: String = ""
    
    // 星期标题
    private let weekdays = ["一", "二", "三", "四", "五", "六", "日"]
    
    private var isFormValid: Bool {
        !location.isEmpty
    }
    
    var body: some View {
        NavigationStack {
            sessionFormContent
                .navigationTitle(navigationTitle)
                .toolbarRole(.editor)
                .toolbar {
                    ToolbarItem(placement: .cancellationAction) {
                        Button("取消") {
                            dismiss()
                        }
                    }
                    
                    ToolbarItem(placement: .confirmationAction) {
                        Button("保存") {
                            saveSession()
                            dismiss()
                        }
                        .disabled(!isFormValid)
                    }
                }
                .onAppear {
                    setupInitialValues()
                }
        }
    }
    
    // Main content view
    private var sessionFormContent: some View {
        ZStack {
            // Background
            (colorScheme == .dark ? Color.black : Color.gray.opacity(0.1))
                .ignoresSafeArea()
            
            // Main content with horizontal layout
            HStack(spacing: 0) {
                // Left side - Form content
                formFieldsSection
                
                // Vertical divider
                Divider()
                    .padding(.vertical)
            }
            .frame(minWidth: 600, idealWidth: 700, maxHeight: .infinity)
        }
    }
    
    // Form fields section
    private var formFieldsSection: some View {
        ScrollView(.vertical) {
            VStack(spacing: 24) {
                // 课时信息
                FormSection(title: "课时信息") {
                    weekdayAndTimeSection
                }
                
                // 位置信息
                FormSection(title: "位置信息") {
                    locationAndClassSection
                }
            }
            .padding()
            .frame(maxWidth: .infinity)
        }
        .frame(minWidth: 300, idealWidth: 350, maxWidth: .infinity)
    }
    
    // Weekday and time slot section
    private var weekdayAndTimeSection: some View {
        VStack(alignment: .leading, spacing: 16) {
            // 星期选择
            HStack {
                Picker("星期", selection: $weekday) {
                    ForEach(1...7, id: \.self) { day in
                        Text("周\(weekdays[day-1])").tag(day)
                    }
                }
                .pickerStyle(.menu)
            }
            
            Divider()
            
            // 节次选择
            HStack {
                Picker("节次", selection: $timeSlot) {
                    ForEach(TimeSlot.defaultSlots) { slot in
                        Text("\(slot.name) (\(slot.timeRange))").tag(slot.id)
                    }
                }
                .pickerStyle(.menu)
            }
        }
        .frame(maxWidth: .infinity)
    }
    
    // Location and class input section
    private var locationAndClassSection: some View {
        VStack(alignment: .leading, spacing: 16) {
            VStack(alignment: .leading, spacing: 8) {
                TextField("上课地点", text: $location)
                    .textFieldStyle(RoundedBorderTextFieldStyle())
                    .autocorrectionDisabled()
                    .frame(maxWidth: .infinity)
                
                Text("请输入教室或上课地点")
                    .font(.caption)
                    .foregroundStyle(.secondary)
            }
            
            Divider()
            
            VStack(alignment: .leading, spacing: 8) {
                TextField("班级", text: $schoolClass)
                    .textFieldStyle(RoundedBorderTextFieldStyle())
                    .autocorrectionDisabled()
                    .frame(maxWidth: .infinity)
                
                Text("可选，请输入班级名称")
                    .font(.caption)
                    .foregroundStyle(.secondary)
            }
        }
        .frame(maxWidth: .infinity)
    }
    
    private var navigationTitle: String {
        switch mode {
        case .add:
            return "添加课时"
        case .edit:
            return "编辑课时"
        }
    }
    
    private func setupInitialValues() {
        switch mode {
        case .add:
            // 使用默认值
            break
        case .edit(let session):
            weekday = session.weekday
            timeSlot = session.timeSlot
            location = session.location
            schoolClass = session.schoolCalss
        }
    }
    
    private func saveSession() {
        switch mode {
        case .add:
            course.addSession(
                weekday: weekday,
                timeSlot: timeSlot,
                location: location,
                schoolClass: schoolClass
            )
            
        case .edit(let session):
            session.weekday = weekday
            session.timeSlot = timeSlot
            session.location = location
            session.schoolCalss = schoolClass
        }
    }
    
    private func timeSlotString(for slot: Int) -> String {
        let slots = TimeSlot.defaultSlots
        if slot >= 1 && slot <= slots.count {
            let timeSlot = slots[slot - 1]
            return "\(timeSlot.name) (\(timeSlot.timeRange))"
        }
        return ""
    }
}

#Preview("添加课时") {
    SessionFormView(mode: .add, course: PreviewData.createSampleCourse())
        .modelContainer(PreviewData.createContainer())
}

#Preview("编辑课时") {
    SessionFormView(mode: .edit(PreviewData.createSampleSession()), course: PreviewData.createSampleCourse())
        .modelContainer(PreviewData.createContainer())
}
